;################################################################### LIBTH.MAC
; Purpose.....:	Library test program, hardware functions
;		To be included by LIBT.MAC
; History.....: 18.10.2023: Created
;#############################################################################

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ Combo Chip
;====================================================================== CITA()
; Purpose.....:	Initialize and run Combo timer A interrupts and callbacks.
;		Timings can be monitored on PIO1, port A
;		Bit0: Mainloop time, triggered on every main loop pass
;		Bit1: Timer interrupt, triggered by reading int elapsed flag 
;		Bit2: Callback1 run
;		Bit3: Callback2 run	
;		A note on interrupt rate: When using BDOS and/or BIOS calls
;		There are 2ms 'pause' every 20mS (probably video-vsync-wait),
;		the mainloop time is therefore >2mS at least every 20mS.
;		When using interrupt rates below the mainloop time, detection
;		in mainloop context gets unreliable. 
;=============================================================================
CITA::		LD	DE,tCITAS
		CALL	gCOSN		; print sign-on message

		LD	A,hcPMOU	; PIO1-A output mode
		CALL	hP1AS		; set it up

		SCF
		CALL	hIS		; setup interrupt table and Z80

		LD	A,hcCT10	; 10mS interval
		SCF			; tell hCTAS to initialize
		CALL	hCTAS		; setup timer

;--------------------------------------------------- setup interrupt callbacks
CITAS::		LD	HL,CITAC1	; address of callback function
		LD	DE,-1		; run infinite
		LD	BC,100		; interval
		CALL	hCTAC		; setup callback

		LD	HL,CITAC3	; LED toggler
		LD	DE,-1		; run forever
		LD	BC,50		; half a second interval
		CALL	hCTAC		; set up callback

		SCF
		CALL	hCTAR		; run the timer
		EI

;--------------------------------------------------------- timer callback loop
CITAL:		CALL	hKR		; read keyboard
		CP	gcESC
		JR	Z,CITAX
		CALL	hCTAB		; callback handler

;-------------------------- main loop time measurement, signal on PIO1-A, bit0
		LD	A,(vCITAP)	; get PIO stats
		XOR	01		; invert bit0
		LD	(vCITAP),A	; write back
		SCF
		CALL	hP1AIO		; update PIO port

;-------------- direct monitor of interrupt elapsed flag, show on PIO1-A, bit1
		LD	A,(hvICTF)	; get elapsed flags
		BIT	hbCTE1,A	; elapsed?
		JR	Z,CITAL		; no, back to loop

		RES	hbCTE1,A	; reset elapsed flag
		LD	(hvICTF),A	; and write back
		LD	A,(vCITAP)	; get PIO stats
		XOR	02		; invert bit 1
		LD	(vCITAP),A	; and write back
		SCF			; do not skip output
		CALL	hP1AIO		; write the data
		JR	CITAL		; and back to loop

;--------------------------------------------- termination, de-init interrupts
CITAX:		XOR	A		; tell hCTAR to stop
		CALL	hCTAR		; stop timer in breakpoint

CITAB::		NOP			; space for breakpoint
		JR	CITAQ		; to skip if continue from breakpoint
		SCF			; resume timer
		CALL	hCTAR		; on continue from breakpoint
		JR	CITAL		; back to loop

CITAQ:		XOR	A		; tell hCTAR to stop
		CALL	hCTAR		; stop timer A
		XOR	A		; tell hIS to de-initialize		
		CALL	hIS		; terminate interrupt operation

		XOR	A		; redisplay menu / LED off
		OUT	(hcKPLD),A	; capslock LED off
		RET

;------------------------------------------------------------------- variables
vCITAP:		DB	0		; PIO status bits

;------------------------------------------------------------- sign-on message
tCITAS:		DB	gcCR,gcLF
		DB	'Combo timer A interrupt, PIO1-A, 0:Mainloop, '
		DB	'1:Intflag, 2:CB1, 3:CB2'
		DB	gcCR,gcLF,gcLF,'$'

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ callback function 1: CITAC1()
; Purpose.....:	Timer callback 1, toggles bit2 of PIO1-A
; Input.......:	All registers undefined
; Labels......: CITAC1: C)ombo I)nterrupt T)imer A) C)allback 1)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CITAC1::	LD	DE,hvVRCL	; CR and clear to end of line
		XOR	A		; code not embedded in string
		CALL	gCOD		; send to terminal
		LD	A,'1'		; signal callback 1
		CALL	gCOC

;----------------------------------------------- start callback2 for five runs
		LD	HL,CITAC2	; address of callback function
		LD	DE,5		; run five times
		LD	BC,10		; interval 100mS
		CALL	hCTAC		; setup callback

;--------------------------------------------------------- toggle PIO1 PA2 bit
		LD	A,(vCITAP)	; A= PIO status
		XOR	04		; invert bit2
		LD	(vCITAP),A	; back to status
		SCF			; do not skip output
		CALL	hP1AIO		; write the data

		RET
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ eoC CITAC1()

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ CITAC2()
; Purpose.....:	Timer callback 2, toggles bit3 on PIO1-A
; Input.......:	All registers undefined
; Labels......: CITAC2: C)ombo I)nterrupt T)imer A) C)allback 2)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CITAC2::	LD	A,'2'		; signal callback 2
		CALL	gCOC

		LD	A,(vCITAP)	; A= PIO status
		XOR	08		; invert bit3
		LD	(vCITAP),A	; and ack to image
		SCF			; do not skip output
		CALL	hP1AIO		; write to PIO

		RET
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ eoC CITAC2()

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ CITAC3()
; Purpose.....:	Timer callback 3, toggles CAPS-LOCK LED
; Input.......:	All registers undefined
; Labels......: CITAC3: C)ombo I)nterrupt T)imer A) C)allback 3)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CITAC3::	LD	A,(vCITA3)	; get LED status
		XOR	01		; toggle bit0
		LD	(vCITA3),A	; and write back

		OUT	(hcKPLD),A	; toggle LED
		RET

;------------------------------------------------------------------- variables
vCITA3:		DB	00		; caps-lock LED status	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ eoF CITAC3()
;================================================================== eoF CITA()

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ SIO-A and B
; For testing SIO-A, replace all occurences of 'SB' with 'SA'
; For testing SIO-B. replace all occurences of 'SA' with 'SB'

		IF	uhSBPW		; if polling read
		IFE	uhSBPR		; and not polling read
;======================================================================= SPW()
; Purpose.....:	Write test pattern to SIO in polling mode
; Requires....:	hSxS(), hSxPWH(), hSxPKS(), hSxWE(), gCOSN(), gCOC()
; Labels......: SPWxx: S)IO P)olling W)rite
;=============================================================================
SPW::		LD	DE,tSPWS
		CALL	gCOSN		; print sign-on message

;----------------------------------------------------- setup SIO and abort key
		LD	A,hcSFHD	; DTR handshake
		LD	B,hcSR9K	; baudrate
		LD	C,1		; use handshake wait mode
		SCF			; initialize
		CALL	hSBS		; setup SIO-A
		XOR	A		; do not wait
		CALL	hSBPWH		; for handshake in hSBPW()
		LD	A,gcESC
		CALL	hSBPKS		; set abort character
		LD	HL,tSPWP	; HL= pointer to pattern

;----------------------------------------------------------- pattern-send loop
SPWL::		LD	A,(HL)		; get char to send
		INC	HL		; point to next
		OR	A		; end of table?
		JR	NZ,SPWW		; no, write char
		LD	HL,tSPWP	; yes, rewind pointer
		JR	SPWL		; and get 1st character

SPWW:		CALL	hSBPW		; send character
		JR	C,SPWX		; C=1 aborted while ready-wait
		JR	NZ,SPWE		; handshake ready, continue
		DEC	HL		; retry with same char again
		LD	A,(HL)		; A= current character
		JR	SPWC		; write to screen
	
SPWE:		CALL	hSBWE		; get write errors
		JR	Z,SPWL		; none, go to next
		LD	A,'#'		; signal for write error

SPWC:		CALL	gCOC		; write to screen
		JR	SPWL		; retry with next char

SPWX::		NOP			; space for breakpoint on exit
		JR	SPWQ		; to skip if continue from breakpoint
		JR	SPWL		; and continue from breakpoint

SPWQ:		XOR	A		; de-init
		CALL	hSBS		; SIO-A
		SCF			; terminate request
		RET

;---------------------------------------------------------------- test pattern
tSPWP:		DB	'SIO SB poll: '
		DB	'ABCDEFGHIJKL'
		DB	'MNOPQRSTUVWX'
		DB	'YZ'
		DB	gcCR,gcLF,0

;------------------------------------------------------------- sign-on message
tSPWS:		DB	gcCR,gcLF
		DB	'SIO SB poll, write'
		DB	gcCR,gcLF,gcLF,'$'
;=================================================================== eoF SPW()
		ENDIF			; uhSBPW
		ENDIF			; /uhSBPR

		IF	uhSBPR		; SIO polling read
;====================================================================== SPRW()
; Purpose.....:	Read data from SIO in polling mode and print on screen.
;		If write is enabled, echo it back to sender
; Requires....:
; Labels......: SPRWx: S)IO P)olling R)ead W)rite
;=============================================================================
SPRW::		LD	DE,tSPRS
		CALL	gCOSN		; print sign-on message

;----------------------------------------------------------------------- setup
		LD	A,hcSFHD	; DTR handshake
		LD	B,hcSR9K	; baudrate
		LD	C,1		; use handshake wait mode
		SCF			; initialize
		CALL	hSBS		; setup SIO-A

		LD	A,hcPMOU	; PIO1-A output mode
		CALL	hP1AS		; set it up

		IF	uhSBPW
		XOR	A		; do not wait
		CALL	hSBPWH		; for handshake in hSBPW()
		ENDIF

		LD	A,gcESC
		CALL	hSBPKS		; set abort character

;------------------------------------------------------------- read/write loop
SPRWL:		CALL	hSBPKC		; check for abort request
		JR	C,SPRWX		; CY=1, terminate

		CALL	hSBPR		; read char from SIO
		JR	Z,SPRWL		; nothing received, loop
		LD	L,A		; L= received char

		JR	NC,SPRWE	; CY=0, no buffer overflow
		LD	A,'$'		; signal for RX buffer overflow
		JR	SPRWC		; show it

SPRWE:		CALL	hSBRE		; get read errors
		LD	A,'*'		; signal for read error
		JR	NZ,SPRWC	; show it
		LD	A,L		; A= received character

;--------------------------------------------- write back to sender if enabled
		IF	uhSBPW
		CALL	hSBPW		; send character
		JR	C,SPRWX		; C=1 aborted while ready-wait
		JR	NZ,SPRWO	; handshake ready, continue
		LD	A,'!'		; signal handshake not ready
		JR	SPRWC		; write to screen
	
SPRWO:		CALL	hSBWE		; get write errors
		LD	A,L		; A= received char
		JR	Z,SPRWC		; none, show it
		LD	A,'#'		; signal for write error
		JR	SPRWC		; show it
		ENDIF			; uhSBPW

SPRWC:		CALL	gCOC		; write to screen
		JR	SPRWL		; back to loop

SPRWX::		NOP			; space for breakpoint on exit
		JR	SPRWQ		; to skip if continue from breakpoint
		JR	SPRWL		; and continue from breakpoint

SPRWQ:		XOR	A		; de-init
		CALL	hSBS		; SIO-A
		SCF			; terminate request
		RET

;------------------------------------------------------------- sign-on message
tSPRS:		DB	gcCR,gcLF
		DB	'SIO SB poll: 9600 baud, DTR/DSR '
		IF	uhSBPR
		DB	'read '
		ENDIF
		IF	uhSBPW
		DB	'write '
		ENDIF
		DB	gcCR,gcLF,gcLF,'$'
;================================================================== eoF SPRW()
		ENDIF			; uhSBPR
		IF	uhSBIW		; TX interrupt	
;======================================================================= SIW()
; Purpose.....:	Write test pattern to SIO in interrupt mode
; Requires....: gCOSN(), gCOC(), gCIB(), hIS(), hSxS(), hSxIW()
; Labels......: SIWxxx: S)IO I)nterrupt W)rite
;=============================================================================
SIW::		LD	DE,tSIWS	; print sign-on message
		CALL	gCOSN
	
;----------------------------------------------- initialize interrupts and SIO
		SCF			; tell hIS to initialize		
		CALL	hIS		; setup for interrupt operation

		LD	A,hcSFHD 	; DTR handshake
		LD	B,hcSR9K	; baudrate	
		SCF			; do init
		CALL	hSBS		; init SIO-A
		LD	HL,tSIWP	; HL= pattern start	
		EI			; enable interrupts
	
;------------------------------------------------------------------ write loop
SIWL:		CALL	gCIB		; read keyboard using BIOS
		CP	gcESC		; abort request?
		JR	Z,SIWX		; yes, terminate

		LD	A,(HL)		; get char to send
		OR	A		; end of pattern?
		JR	NZ,SIWW
		LD	HL,tSIWP	; yes, rewind pointer
		LD	A,(HL)		; and load 1st char

SIWW:		CALL	hSBIW		; write to SIO-A buffer
		INC	HL		; point to next char to send
		JR	NC,SIWL		; buffer not full, go for next
		DEC	HL		; buffer full, try with same again
		LD	A,(HL)		; print unsent character to console
		CP	gcCR		; except for CR
		JR	Z,SIWL
		CP	gcLF		; and LF
		JR	Z,SIWL
		CALL	gCOC
		JR	SIWL		; retry same char

;------------------------------------- termination, de-init SIO and interrupts
SIWX:		NOP			; space for breakpoint
		JR	SIWQ		; to skip if continue from breakpoint
		JR	SIWL		; and continue from breakpoint

SIWQ:		XOR	A		; terminate
		CALL	hSBS		; de-init SIO-A
		XOR	A		; tell hIS to de-initialize		
		CALL	hIS		; terminate interrupt operation

		XOR	A		; redisplay menu
		RET

;---------------------------------------------------------------- test pattern
tSIWP:		DB	'SIO SB int: '
		DB	'abcdefghijklmnopqrstuvwxyz'
		DB	gcCR, gcLF, 0

;------------------------------------------------------------- sign-on message
tSIWS:		DB	gcCR,gcLF
		DB	'SIO SB interrupt write'
		DB	gcCR,gcLF,gcLF,'$'
;=================================================================== eoF SIW()
		ENDIF			; uhSBIW
		IF	uhSBIR		; SIO interrupt read
;====================================================================== SIRW()
; Purpose.....:	Read data from SIO in interrupt mode and print on screen.
;		If write is enabled, echo it back to sender
;		If combo timer is enabled, do simultaneous interrupts
; Requires....:
; Labels......: SIRWxx: S)IO I)nterrupt R)ead W)rite
;=============================================================================
SIRW::		LD	DE,tSIRWS	; print sign-on message
		CALL	gCOSN

		SCF			; tell hIS() to initialize
		CALL	hIS		; setup for interrupt operation

		LD	A,hcSFHD 	; DTR handshake
		LD	B,hcSR9K	; baudrate	
		SCF			; do init
		CALL	hSBS		; init SIO

;-------------------------- simultaneous operation of timer and SIO interrupts
		IF	uhCITA		
		LD	A,hcCT10	; 10mS interval
		SCF			; tell hCTAS to initialize
		CALL	hCTAS		; setup timer
		LD	HL,CITAC3	; address of callback function
		LD	DE,-1		; run infinite
		LD	BC,20		; interval 200mS
		CALL	hCTAC		; setup callback
		SCF
		CALL	hCTAR		; run the timer

		LD	A,hcPMOU	; PIO1-A output mode for monitor
		CALL	hP1AS		; set it up
		ENDIF			; uhCITA

		EI			; enable interrupts
		
;----------------------------------- check keyboard for abort or break request
SIRWL:		CALL	gCIB		; read keyboard using BIOS
		CP	gcESC		; abort request?
		JR	Z,SIRWX		; yes, terminate

;------------------------------------------- combo timer-A callback if enabled
		IF	uhCITA
		CALL	hCTAB		; timer callback handler
		ENDIF			; uhCITA

;--------------------------------------------- read byte from interrupt buffer
		CALL	hSBIR		; read byte from int buffer SIO
		LD	L,A		; save it for display
		LD	A,'*'		; preset to overflow
		JR	C,SIRWC		; C=1, buffer overflow
		JR	Z,SIRWL		; nothing ready, reread

;------------------------- if interrupt write enabled, echo received character
		LD	A,L		; recall read character

		IF 	uhSBIW		; if interrupt write 
		CALL	hSBIW		; write to SIO
		LD	A,'#'		; preset to overflow
		JR	C,SIRWC		; signal tx buffer overflow
		LD	A,L		; A= read byte
		ENDIF

SIRWC:		CALL	gCOC		; and display it
		JR	SIRWL		; back to loop

;------------------------------------- termination, de-init SIO and interrupts
SIRWX:		NOP			; space for breakpoint
		JR	SIRWQ		; to skip if continue from breakpoint
		JR	SIRWL		; and continue from breakpoint

SIRWQ:		XOR	A		; terminate
		CALL	hSBS		; de-init SIO-A

		IF	uhCITA		; if combo timer
		XOR	A		; de-init it
		CALL	hCTAS
		ENDIF			; uhCITA

		XOR	A		; tell hIS to de-initialize		
		CALL	hIS		; terminate interrupt operation

		XOR	A		; redisplay menu
		OUT	(hcKPLD),A	; capslock LED off
		RET

;------------------------------------------------------------- sign-on message
tSIRWS:		DB	gcCR,gcLF
		DB	'SIO SB int: 9600 baud, DTR/DSR'
		IF	uhSBIR
		DB	', read'
		ENDIF
		IF	uhSBIW
		DB	', write'
		ENDIF
		IF	uhCITA
		DB	', Combo Timer-A interrupts'
		ENDIF
		DB	gcCR,gcLF,gcLF,'$'
;================================================================== eoF SIRW()
		ENDIF			; uhSBIR

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ PIO1
;======================================================================== P1()
; Purpose.....:	PIO1 out/input in mode 1
; Requires....:	hSxS(), hSxPWH(), hSxPKS(), hSxWE(), gCOSN(), gCOC()
; Labels......: P1xxxx: P)IO 1)
;=============================================================================
P1::		LD	DE,tP1S
		CALL	gCOSN		; print sign-on message

		LD	DE,hvVCOF	; cursor off
		XOR	A		; not embedded in string
		CALL	gCOD		; print code	

		LD	A,hcPMOU	; PIO1-A output mode
		CALL	hP1AS		; set it up
		LD	A,hcPMIN	; PIO1-B input mode
		CALL	hP1BS		; set it up
		LD	B,0		; output counter

;----------------------------------- check keyboard for abort or break request
P1L:		CALL	hKR		; read keyboard using BIOS
		CP	gcESC		; abort request?
		JR	Z,P1X		; yes, terminate

;------------------------------------------------------ write/read to/from PIO
		LD	A,B		; A= out value
		INC	B		; bump counter
		SCF			; write data
		CALL	hP1AIO		; to PIO port A

		XOR	A		; skip write
		CALL	hP1BIO		; read from port B

;------------------------------------------------------- display in/out values
		PUSH	AF		; save input value
		LD	HL,vP1O		; HL= store address
		LD	D,0
		LD	E,B		; DE= out value
		LD	C,2		; two digits
		CALL	gOBH		; convert to HEX-ASCII
		POP	AF		; recall input value

		LD	HL,vP1I		; HL= store address
		LD	D,0
		LD	E,A		; DE= in value
		LD	C,2		; two digits
		CALL	gOBH		; convert to HEX-ASCII

		LD	DE,tP1IO	; string address
		CALL	gCOSN		; print it
		JR	P1L		; go to loop

;----------------------------------------------------------------- termination
P1X:		NOP			; space for breakpoint
		JR	P1Q		; to skip if continue from breakpoint
		JR	P1L		; and continue from breakpoint

P1Q:		LD	DE,hvVCON	; cursor on
		XOR	A		; not embedded in string
		CALL	gCOD		; print code	

		XOR	A		; redisplay menu
		RET

;--------------------------------------------------------- output/input values
tP1IO:		DB	gcCR
		DB	'Out: '
vP1O:		DB	'--'
		DB	'  In: '
vP1I:		DB	'--'
		DB	'$'

;------------------------------------------------------------- sign-on message
tP1S:		DB	gcCR,gcLF
		DB	'PIO1 Port A output, Port B input'
		DB	gcCR,gcLF,gcLF,'$'

;==================================================================== eoF P1()

;############################################################### EOF LIBTH.INC
================================== eoF P1()

;####################